@@ -1,5 +1,5 @@
 #
-# Copyright 2017-2019 European Centre for Medium-Range Weather Forecasts (ECMWF).
+# Copyright 2017-2020 European Centre for Medium-Range Weather Forecasts (ECMWF).
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -43,7 +43,15 @@ class RaiseOnAttributeAccess(object):
         raise RuntimeError(self.message) from self.exc
 
 
-for libname in ['eccodes', 'libeccodes.so', 'libeccodes']:
+LIBNAMES = ["eccodes", "libeccodes.so", "libeccodes"]
+
+try:
+    import ecmwflibs
+    LIBNAMES.insert(0, ecmwflibs.find("eccodes"))
+except Exception:
+    pass
+
+for libname in LIBNAMES:
     try:
         lib = ffi.dlopen(libname)
         LOG.info("ecCodes library found using name '%s'.", libname)
